function [s_d, s_l, s_gau, s_log] =  SMSE(imo, imd)

% SMSE - measure the distortion degree of distorted image 'imd' with the reference image 'imo'
% using the Structure MSE.
% 
% inputs:
% 
% imo - the reference image (grayscale image, double type, 0~255)
% imd - the distorted image (grayscale image, double type, 0~255)
% 
% outputs:

% s_d: the distortion scores obtained by SMSE using the difference operator
% s_l: the distortion scores obtained by SMSE using the Laplace operator
% s_gau: the distortion scores obtained by SMSE using the Gaussian gradient operator
% s_log: the distortion scores obtained by SMSE using the Laplacian of Gaussian (LOG) operator

% This is an implementation of the SMSE algorithm in the following paper:
% W. Xue, X. Mou, L. Zhang, and X. Feng, "Perceptual Fidelity Aware Mean Squared Error," In ICCV 2013.

E_map = imo-imd;
MSE = mean2(E_map.^2);

%%% SMSE with Sd %%%%%%
dx = [1 -1];
dy = dx';

IxErr = filter2(dx,E_map,'same');
IyErr = filter2(dy,E_map,'same');
Hdiff_MSE = mean2(IxErr.^2+IyErr.^2);
beta2 =  8; % |beta_max^2| for the difference operator
s_d = MSE-1/beta2*Hdiff_MSE;

%%% SMSE with Sl %%%%%%
laplace = [0 1 0;1 -4 1; 0 1 0];

lapE = filter2(laplace,E_map,'same');
Hlap_MSE = mean2(lapE.^2);
beta2 = 64; % |beta_max^2| for the Laplace operator
s_l = MSE-1/beta2*Hlap_MSE;


%%% SMSE with S_gau %%%

sigma = 0.5;
[IxErr, IyErr] = gaussian_derivative(E_map,sigma);
Hgau_MSE = mean2(IxErr.^2+IyErr.^2);
beta2 = 1.5315; %|beta_max^2| the gaussian derivative operators
s_gau = MSE-1/beta2*Hgau_MSE;


%%% SMSSE with LOG %%%

sigma = 0.5;
hlog = fspecial('log', (2*ceil(3*sigma)+1)*[1 1],sigma);
LogErr = conv2(E_map,hlog,'valid');
Hlog_MSE = mean2(LogErr.^2);
beta2 = 41.8833;
s_log = MSE-1/beta2*Hlog_MSE;

function [gx,gy] = gaussian_derivative(imd,sigma)
window1 = fspecial('gaussian',2*ceil(3*sigma)+1+2, sigma);
window1 = window1/sum(window1(:));
winx = window1(2:end-1,2:end-1)-window1(2:end-1,3:end);
winy = window1(2:end-1,2:end-1)-window1(3:end,2:end-1);
gx = filter2(winx,imd,'same');
gy = filter2(winy,imd,'same');